----------------------------------------------------------------------------------------------------
--Author: Falkkor-----------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
--"Imitation is the sincerest form of flattery that mediocrity can pay to greatness."---------------
----------------------------------------------------------------------------------------------------

xtremeSpeedUnlocker = {};
xtremeSpeedUnlocker.modDirectory  = g_currentModDirectory;
local modDescFile = loadXMLFile("modDesc", xtremeSpeedUnlocker.modDirectory .. "modDesc.xml");
xtremeSpeedUnlocker.title = getXMLString(modDescFile, "modDesc.title.en");
xtremeSpeedUnlocker.author = getXMLString(modDescFile, "modDesc.author");
xtremeSpeedUnlocker.version = getXMLString(modDescFile, "modDesc.version");
delete(modDescFile);

function xtremeSpeedUnlocker.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations)
		and SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function xtremeSpeedUnlocker.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLeaveVehicle", xtremeSpeedUnlocker);
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", xtremeSpeedUnlocker);
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", xtremeSpeedUnlocker);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", xtremeSpeedUnlocker);
	SpecializationUtil.registerEventListener(vehicleType, "onDraw", xtremeSpeedUnlocker);
end;

function xtremeSpeedUnlocker.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "setXtremeSpeedUnlockerEventState",  xtremeSpeedUnlocker.setXtremeSpeedUnlockerEventState);
end;

function xtremeSpeedUnlocker.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getSpeedLimit", xtremeSpeedUnlocker.getSpeedLimit);
end;

function xtremeSpeedUnlocker:getSpeedLimit(superFunc, onlyIfWorking) --server side
    local limit = math.huge;
    local doCheckSpeedLimit = self:doCheckSpeedLimit();
    if onlyIfWorking == nil or (onlyIfWorking and doCheckSpeedLimit) then
		if self.spec_xtremeSpeedUnlocker == nil or (self.spec_xtremeSpeedUnlocker ~= nil and not self.spec_xtremeSpeedUnlocker.active) then
			limit = self.speedLimit;
        end;
        local damage = self:getVehicleDamage();
        if damage > 0 then
            limit = limit * (1 - damage * Vehicle.DAMAGED_SPEEDLIMIT_REDUCTION);
        end;
    end;

	if self.spec_xtremeSpeedUnlocker ~= nil and self.spec_xtremeSpeedUnlocker.active then 
		local speedLimit = self.spec_xtremeSpeedUnlocker.baseVehicleSpeedLimit;
		if speedLimit ~= nil and speedLimit > 999 then speedLimit = nil; end;
		self.spec_xtremeSpeedUnlocker.suggestedLimit = speedLimit;
	end;

	local attachedImplements;
	if self.getAttachedImplements ~= nil then
		attachedImplements = self:getAttachedImplements();
	end;

	if attachedImplements ~= nil then
		for _, implement in pairs(attachedImplements) do
			if implement.object ~= nil then
				local speed, implementDoCheckSpeedLimit = implement.object:getSpeedLimit(onlyIfWorking);
				if onlyIfWorking == nil or (onlyIfWorking and implementDoCheckSpeedLimit) then
					if self.spec_xtremeSpeedUnlocker == nil or (self.spec_xtremeSpeedUnlocker ~= nil and not self.spec_xtremeSpeedUnlocker.active) then
						limit = math.min(limit, speed);
					end;
				end;
				doCheckSpeedLimit = doCheckSpeedLimit or implementDoCheckSpeedLimit;
				if self.spec_xtremeSpeedUnlocker ~= nil and self.spec_xtremeSpeedUnlocker.active then
					local speedLimit = implement.object.speedLimit;
					if speedLimit ~= nil and speedLimit > 999 then speedLimit = nil; end;
					if self.spec_xtremeSpeedUnlocker.suggestedLimit == nil and speedLimit ~= nil then
						self.spec_xtremeSpeedUnlocker.suggestedLimit = speedLimit;
					elseif speedLimit ~= nil then 
						self.spec_xtremeSpeedUnlocker.suggestedLimit = math.min(self.spec_xtremeSpeedUnlocker.suggestedLimit, speedLimit);
					end;
				end;
			end;
		end;
	end;

    return limit, doCheckSpeedLimit;
end;

function xtremeSpeedUnlocker:onLoad()
	self.spec_xtremeSpeedUnlocker = {};
	self.spec_xtremeSpeedUnlocker.active = false;
	self.spec_xtremeSpeedUnlocker.events = {};
	self.spec_xtremeSpeedUnlocker.eventIds = {};
	self.spec_xtremeSpeedUnlocker.suggestedLimit = nil;
	self.spec_xtremeSpeedUnlocker.baseVehicleSpeedLimit = getXMLFloat(self.xmlFile.handle,"vehicle.base.speedLimit#value");
end;

function xtremeSpeedUnlocker:onUpdate()
	if not self.isServer then--client side only (not called by single player or the host in non-dedicated servers)
		if self.spec_xtremeSpeedUnlocker ~= nil and self.spec_xtremeSpeedUnlocker.active then
			local isControlled = self.getIsControlled ~= nil and self:getIsControlled();
			if isControlled or self:getIsActiveForInput(true, true) then
				local speedLimit = self.spec_xtremeSpeedUnlocker.baseVehicleSpeedLimit;
				if speedLimit ~= nil and speedLimit > 999 then speedLimit = nil; end;
				self.spec_xtremeSpeedUnlocker.suggestedLimit = speedLimit;
				local attachedImplements;
				if self.getAttachedImplements ~= nil then
					attachedImplements = self:getAttachedImplements();
				end;
				if attachedImplements ~= nil then
					for _, implement in pairs(attachedImplements) do
						if implement.object ~= nil then
							local speedLimit = implement.object.speedLimit;
							if speedLimit ~= nil and speedLimit > 999 then speedLimit = nil; end;
							if self.spec_xtremeSpeedUnlocker.suggestedLimit == nil and speedLimit ~= nil then
								self.spec_xtremeSpeedUnlocker.suggestedLimit = speedLimit;
							elseif speedLimit ~= nil then 
								self.spec_xtremeSpeedUnlocker.suggestedLimit = math.min(self.spec_xtremeSpeedUnlocker.suggestedLimit, speedLimit);
							end;
						end;
					end;
				end;
			end;
		end;
	end;
end;

function xtremeSpeedUnlocker:onLeaveVehicle()
	self.spec_xtremeSpeedUnlocker.suggestedLimit = nil;
end;

 function xtremeSpeedUnlocker:setXtremeSpeedUnlockerEventState(state, noEventSend)
    xtremeSpeedUnlockerEvent.sendEvent(self, state, noEventSend);
	self.spec_xtremeSpeedUnlocker.active = state;
	if not self.spec_xtremeSpeedUnlocker.active then
		self.spec_xtremeSpeedUnlocker.suggestedLimit = nil;
	end;
end;

function xtremeSpeedUnlocker:toggleActive()
	self:setXtremeSpeedUnlockerEventState(not self.spec_xtremeSpeedUnlocker.active);
end;

function xtremeSpeedUnlocker:onDraw(dt)
	if self.spec_xtremeSpeedUnlocker.active then
		if self.spec_xtremeSpeedUnlocker.suggestedLimit ~= nil then
			local speed = tostring(math.floor(self.spec_xtremeSpeedUnlocker.suggestedLimit));
			g_currentMission:addExtraPrintText("Recommended Operating Speed: "..speed);
		end;
	end;
end;

function xtremeSpeedUnlocker:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.isClient then
		self:clearActionEventsTable(self.spec_xtremeSpeedUnlocker.events);
		self.spec_xtremeSpeedUnlocker.eventIds = {};
        if self:getIsActiveForInput(true, true) then
			local _, actionEventId = self:addActionEvent(self.spec_xtremeSpeedUnlocker.events, 'xtremeSpeedUnlocker_toggleKey', self, xtremeSpeedUnlocker.toggleActive, false, true, false, true, nil);
			g_inputBinding:setActionEventTextVisibility(actionEventId, true);
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_HIGH);
			table.insert(self.spec_xtremeSpeedUnlocker.eventIds, actionEventId);
		end;
	end;
end;

--Vehicle.getSpeedLimit = Utils.overwrittenFunction(Vehicle.getSpeedLimit, xtremeSpeedUnlocker.getSpeedLimit);

xtremeSpeedUnlockerEvent = {};
xtremeSpeedUnlockerEvent_mt = Class(xtremeSpeedUnlockerEvent, Event);
InitEventClass(xtremeSpeedUnlockerEvent,"xtremeSpeedUnlockerEvent");

function xtremeSpeedUnlockerEvent:emptyNew()
    local self = Event.new(xtremeSpeedUnlockerEvent_mt);
    self.className = "xtremeSpeedUnlockerEvent";
    return self;
end;
function xtremeSpeedUnlockerEvent:new(vehicle,state)
	local self = xtremeSpeedUnlockerEvent.emptyNew()
    self.vehicle = vehicle;
    self.state = state;
    return self;
end;
function xtremeSpeedUnlockerEvent:readStream(streamId,connection)
    self.vehicle = NetworkUtil.readNodeObject(streamId);
    self.state = streamReadBool(streamId);
    self:run(connection);
end;
function xtremeSpeedUnlockerEvent:writeStream(streamId,connection)
    NetworkUtil.writeNodeObject(streamId,self.vehicle);
    streamWriteBool(streamId,self.state);
end;
function xtremeSpeedUnlockerEvent:run(connection)
    self.vehicle:setXtremeSpeedUnlockerEventState(self.state, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(xtremeSpeedUnlockerEvent:new(self.vehicle,self.state),nil,connection,self.vehicle);
    end;
end;
function xtremeSpeedUnlockerEvent.sendEvent(vehicle,state,noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(xtremeSpeedUnlockerEvent:new(vehicle,state),nil,nil,vehicle);
        else
            g_client:getServerConnection():sendEvent(xtremeSpeedUnlockerEvent:new(vehicle,state));
        end;
    end;
end;